/*************************************************
* IF Scheme Source File                          *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/if_algo.h>
#include <botan/numthry.h>

namespace Botan {

/*************************************************
* IF_Scheme_PublicKey Initializer                *
*************************************************/
void IF_Scheme_PublicKey::if_initialize(const BigInt& mod, const BigInt& exp,
                                        bool odd_exp_only)
   {
   if(mod <= 15 || mod % 2 == 0)
      throw Invalid_Argument(algo_name() + ": Invalid public modulus");
   if(exp <= 1 || (odd_exp_only  && exp % 2 == 0) ||
                  (!odd_exp_only && exp % 2 == 1))
      throw Invalid_Argument(algo_name() + ": Invalid public exponent");
   n = mod;
   e = exp;
   powermod_e_n = FixedExponent_Exp(e, n);
   odd_exponent = odd_exp_only;
   }

/*************************************************
* IF Scheme Private Key Operation                *
*************************************************/
BigInt IF_Scheme_PrivateKey::if_private_op(const BigInt& i) const
   {
   if(i >= n || i.is_negative())
      throw Invalid_Argument(algo_name() + "::private_op: i >= n || i < 0");
   BigInt j1 = powermod_d1_p(i);
   BigInt j2 = powermod_d2_q(i);
   BigInt h = powermod_d1_p.reduce(sub_mul(j1, j2, c));
   return mul_add(h, q, j2);
   }

/*************************************************
* IF Scheme Precomputation                       *
*************************************************/
void IF_Scheme_PrivateKey::precompute()
   {
   if(n == 0)
      throw Internal_Error(algo_name() + ": precompute called with modulus=0");
   d1 = d % (p - 1);
   d2 = d % (q - 1);
   c = inverse_mod(q, p);
   powermod_d1_p = FixedExponent_Exp(d1, p);
   powermod_d2_q = FixedExponent_Exp(d2, q);
   if(!powermod_e_n.initialized())
      powermod_e_n = FixedExponent_Exp(e, n);
   }

/*************************************************
* Check IF Scheme Private Paramaters             *
*************************************************/
bool IF_Scheme_PrivateKey::check_params() const
   {
   if(p * q != n)
      return false;
   if(!check_prime(p) || !check_prime(q))
      return false;
   return true;
   }

}
