/*************************************************
* X.509 Source File                              *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/x509.h>
#include <botan/filters.h>
#include <botan/asn1.h>
#include <botan/pem.h>

namespace Botan {

/*************************************************
* DER or PEM encode a X.509 public key           *
*************************************************/
void X509_PublicKey::X509_encode(Pipe& pipe, X509_Encoding encoding) const
   {
   if(encoding == PEM)
      pipe.write(PEM_encode());
   else
      {
      SecureVector<byte> encoding = DER_encode();
      pipe.write(encoding, encoding.size());
      }
   }

/*************************************************
* DER encode a X.509 public key                  *
*************************************************/
SecureVector<byte> X509_PublicKey::DER_encode() const
   {
   DER_Encoder encoder;

   encoder.start_sequence();
   encoder.start_sequence();
   DER::encode(encoder, get_oid());
   encoder.add_raw_octets(encode_algo_params());
   encoder.end_sequence();
   DER::encode(encoder, encode_key_x509(), BIT_STRING);
   encoder.end_sequence();
   return encoder.get_contents();
   }

/*************************************************
* PEM encode a X.509 public key                  *
*************************************************/
std::string X509_PublicKey::PEM_encode() const
   {
   SecureVector<byte> encoding = DER_encode();
   return pem_encode(encoding, encoding.size(), "PUBLIC KEY");
   }

/*************************************************
* Decode a BER or PEM encoded X.509 public key   *
*************************************************/
void X509_PublicKey::X509_decode(DataSource& source, X509_Encoding encoding)
   {
   if(encoding == PEM)
      PEM_decode(source);
   else
      BER_decode(source);
   }

/*************************************************
* Decode a BER encoded X.509 public key          *
*************************************************/
void X509_PublicKey::BER_decode(DataSource& source)
   {
   OID oid;
   SecureVector<byte> params, x509_key;

   try
      {
      BER_Decoder decoder(source);
      BER_Decoder outer_sequence = BER::get_subsequence(decoder);
      BER_Decoder inner_sequence = BER::get_subsequence(outer_sequence);

      BER::decode(inner_sequence, oid);
      params = inner_sequence.get_remaining();
      inner_sequence.verify_end();

      BER::decode(outer_sequence, x509_key, BIT_STRING);
      outer_sequence.verify_end();
      }
   catch(Decoding_Error)
      {
      throw Decoding_Error("X.509 public key decoding failed");
      }

   if(!valid_oid(oid))
      throw Decoding_Error("X509: OID mismatch, found " + oid.as_string() +
                           ", expected " + get_oid().as_string());

   Pipe output;
   output.process_msg(params);
   decode_algo_params(output);
   output.process_msg(x509_key);
   output.set_default_msg(1);
   decode_key_x509(output);
   }

/*************************************************
* Decode a PEM encoded X.509 public key          *
*************************************************/
void X509_PublicKey::PEM_decode(DataSource& source)
   {
   Pipe ber;
   ber.process_msg(pem_decode_check_label(source, "PUBLIC KEY"));
   BER_decode(ber);
   }

/*************************************************
* Return the default NULL algorithm paramaters   *
*************************************************/
SecureVector<byte> X509_PublicKey::encode_algo_params() const
   {
   DER_Encoder encoder;
   DER::encode_null(encoder);
   return encoder.get_contents();
   }

/*************************************************
* Check to make sure the paramaters are NULL     *
*************************************************/
void X509_PublicKey::decode_algo_params(DataSource& source)
   {
   BER_Decoder decoder(source);
   BER::decode_null(decoder);
   }

/*************************************************
* Default OID check                              *
*************************************************/
bool X509_PublicKey::valid_oid(const OID& oid) const
   {
   if(oid == get_oid())
      return true;
   return false;
   }

}
